@@ -1,30 +1,3 @@
-#-----------------------------------------------------------------------------
-# Name:        guitools.py
-# Purpose:     GUI utilities
-#
-# Author:      Andras Wacha
-#
-# Created:     2010/02/22
-# RCS-ID:      $Id: guitools.py $
-# Copyright:   (c) 2010
-# Licence:     GPLv2
-#-----------------------------------------------------------------------------
-#       This program is free software; you can redistribute it and/or modify
-#       it under the terms of the GNU General Public License as published by
-#       the Free Software Foundation; either version 2 of the License, or
-#       (at your option) any later version.
-#       
-#       This program is distributed in the hope that it will be useful,
-#       but WITHOUT ANY WARRANTY; without even the implied warranty of
-#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#       GNU General Public License for more details.
-#       
-#       You should have received a copy of the GNU General Public License
-#       along with this program; if not, write to the Free Software
-#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-#       MA 02110-1301, USA.
-
-
 import types
 import numpy as np
 import pylab
@@ -324,11 +297,11 @@ def plot2dmatrix(A,maxval=None,mask=None,header=None,qs=[],showqscale=True,conto
                    color='white',linewidth=3)
         pylab.gca().axis(a)
     if header is not None:
-        pylab.title("#%s: %s" % (header['FSN'], header['Title']))
+        pylab.title("#%d: %s" % (header['FSN'], header['Title']))
     if crosshair and header is not None:
         a=pylab.gca().axis()
-        pylab.plot([extent1[2],extent1[3]],[bcrow,bcrow],'-',color='white')
-        pylab.plot([bccol,bccol],[extent1[0],extent1[1]],'-',color='white')
+        pylab.plot([extent1[0],extent1[1]],[bcrow,bcrow],'-',color='white')
+        pylab.plot([bccol,bccol],[extent1[2],extent1[3]],'-',color='white')
         pylab.gca().axis(a)
 def makemask(mask,A,savefile=None):
     """Make mask matrix.
@@ -682,8 +655,8 @@ def basicfittinggui_old(data,title='',blocking=False):
     return listoffits
         
 def basicfittinggui(data,title='',blocking=False):
-    """Graphical user interface to carry out basic (Guinier, Porod) fitting
-    to 1D scattering data.
+    """Graphical user interface to carry out basic (Guinier, Porod, etc.)
+    fitting to 1D scattering data.
     
     Inputs:
         data: 1D dataset
@@ -699,124 +672,90 @@ def basicfittinggui(data,title='',blocking=False):
     
     if not isinstance(data,utils.SASDict):
         return basicfittinggui_old(data,title,blocking)
+    else:
+        return data.basicfittinggui(title,blocking)
+
+
+def testsmoothing(x,y,smoothing=1e-5,slidermin=None,slidermax=None,
+                  xscaling='linear',yscaling='log',smoothingmode='spline',
+                  callback=None,returnsmoothed=False):
+    """Tool to test and adjust smoothing of a curve.
+    
+    Inputs:
+        x: abscissa
+        y: ordinate
+        smoothing: initial value of the smoothing parameter
+        slidermin: minimum value of the slider. If None, defaults to 10^(log10(smoothing)-2)
+        slidermax: maximum value of the slider. If None, defaults to 10^(log10(smoothing)+2)
+        xscaling: scaling ('log' or 'linear') of the abscissa
+        yscaling: scaling ('log' or 'linear') of the ordinate
+        smoothingmode: mode of smoothing (see fitting.smoothcurve() for valid
+            entries)
+        callback: either None (do nothing) or a callable, accepting three
+            arguments (the smoothing parameter, the smoothed curve and the axes)
+            to do something, even plot.
+        returnsmoothed: if the smoothed curve is to be returned [False by default]            
+    Outputs: s, [smoothedy]
+        s: the final smoothing parameter
+        smoothedy: the smoothed curve (if returnsmoothed was True)
         
-    
-    listoffits=[]
-    
-    leftborder=0.05
-    topborder=0.9
-    bottomborder=0.1
-    leftbox_end=0.3
-    fig=pylab.figure()
-    pylab.clf()
-    plots=[{'name':'Guinier','transform':utils.SASTransformGuinier(),
-            'plotmethod':'plot'},
-           {'name':'Guinier thickness','transform':utils.SASTransformGuinier(2),
-            'plotmethod':'plot'},
-           {'name':'Guinier cross-section','transform':utils.SASTransformGuinier(1),
-            'plotmethod':'plot'},
-           {'name':'Porod','transform':utils.SASTransformPorod(2),
-            'plotmethod':'plot'},
-           {'name':'Double linear','transform':utils.SASTransformLogLog(False,False),
-            'plotmethod':'plot'},
-           {'name':'Logarithmic y','transform':utils.SASTransformLogLog(False,False),
-            'plotmethod':'semilogy'},
-           {'name':'Logarithmic x','transform':utils.SASTransformLogLog(False,False),
-            'plotmethod':'semilogx'},
-           {'name':'Double logarithmic','transform':utils.SASTransformLogLog(False,False),
-            'plotmethod':'loglog'}]
-    buttons=[{'name':'Guinier','fitmethod':'guinierfit'},
-             {'name':'Guinier thickness','fitmethod':'guinierthicknessfit'},
-             {'name':'Guinier cross-section','fitmethod':'guiniercrosssectionfit'},
-             {'name':'Porod','fitmethod':'porodfit'},
-             {'name':'A * q^B','fitmethod':'powerlawfit'},
-             {'name':'A * q^B + C','fitmethod':'powerlawconstantbackgroundfit'},
-             {'name':'A * q^B + C + D * q','fitmethod':'powerlawlinearbackgroundfit'}]
-            
-    for i in range(len(buttons)):
-        ax=pylab.axes((leftborder,topborder-(i+1)*(0.8)/(len(buttons)+len(plots)),leftbox_end,0.7/(len(buttons)+len(plots))))
-        but=matplotlib.widgets.Button(ax,buttons[i]['name'])
-        def onclick(A=None,B=None,data=data,type=buttons[i]['name'],fitfun=buttons[i]['fitmethod']):
-            data1=data.trimzoomed()
-            data1.transform=data.transform
-            pylab.figure()
-            res=getattr(data1,fitfun).__call__(plot=True)
-            listoffits.append({'type':type,'res':res,'time':time.time(),'qmin':data1.q.min(),'qmax':data1.q.max()})
-            pylab.gcf().show()
-        but.on_clicked(onclick)
-    ax=pylab.axes((leftborder,topborder-(len(buttons)+len(plots))*(0.8)/(len(buttons)+len(plots)),leftbox_end,0.7/(len(buttons)+len(plots))*len(plots) ))
-    pylab.title('Plot types')
-    rb=matplotlib.widgets.RadioButtons(ax,[p['name'] for p in plots],active=7)
-    pylab.gcf().blocking=blocking
-    if pylab.gcf().blocking: #if blocking mode is desired, put a "Done" button.
-        ax=pylab.axes((leftborder,0.03,leftbox_end,bottomborder-0.03))
-        b=matplotlib.widgets.Button(ax,"Done")
-        fig=pylab.gcf()
-        fig.fittingdone=False
-        def onclick1(A=None,B=None,fig=fig):
-            fig.fittingdone=True
-            pylab.gcf().blocking=False
-     #       print "blocking should now end"
-        b.on_clicked(onclick1)
-    pylab.axes((0.45,bottomborder,0.5,0.8))
-    def onselectplottype(plottype,q=data['q'],I=data['Intensity'],title=title):
-        pylab.cla()
-        plotinfo=[d for d in plots if d['name']==plottype][0]
-        data.transform=plotinfo['transform']
-        getattr(data,plotinfo['plotmethod']).__call__()
-        pylab.title(title)
-        pylab.gcf().show()
-    rb.on_clicked(onselectplottype)
-    pylab.title(title)
-    data.loglog('.')
-    pylab.gcf().show()
-    pylab.draw()
+    Notes:
+        a matplotlib figure will be presented with a slider to allow the user to
+        change the smoothing parameter.
+    """
+    #create the slider above
     fig=pylab.gcf()
-    while blocking:
-        fig.waitforbuttonpress()
-#        print "buttonpress"
-        if fig.fittingdone:
-            blocking=False
-            #print "exiting"
-    #print "returning"
-    return listoffits
-
-
-#data quality tools
-def testsmoothing(x,y,smoothing=1e-5,slidermin=1e-6,slidermax=1e-2):
-    ax=pylab.axes((0.2,0.85,0.7,0.05));
+    fig.clf()
+    if slidermin is None:
+        slidermin=np.power(10,np.log10(smoothing)-2)
+    if slidermax is None:
+        slidermax=np.power(10,np.log10(smoothing)+2)
+    ax=fig.add_axes((0.3,0.85,0.6,0.05));
     sl=matplotlib.widgets.Slider(ax,'',np.log10(slidermin),np.log10(slidermax),np.log10(smoothing));
-    fig=pylab.gcf()
     fig.smoothingdone=False
-    ax=pylab.axes((0.1,0.85,0.1,0.05));
-    def butoff(a=None):
-        pylab.gcf().smoothingdone=True
+    ax=fig.add_axes((0.1,0.85,0.1,0.05));
+    def butoff(a=None,fig=fig):
+        """This is going to be called if the OK button is pressed"""
+        fig.smoothingdone=True
+        fig.cancel=False
     but=matplotlib.widgets.Button(ax,'Ok')
     but.on_clicked(butoff)
-    pylab.axes((0.1,0.1,0.8,0.7))
-    pylab.cla()
-    pylab.plot(x,y,'.')
-    smoothing=pow(10,sl.val);
-    y1=fitting.smoothcurve(x,y,smoothing,mode='spline')
-    pylab.plot(x,y1,linewidth=2)
-    def fun(a):
-        ax=pylab.axis()
-        pylab.cla()
-        pylab.plot(x,y,'.')
-        smoothing=pow(10,sl.val);
-        y1=fitting.smoothcurve(x,y,smoothing,mode='spline')
-        pylab.plot(x,y1,linewidth=2)
-        pylab.axis(ax)
+
+    ax=fig.add_axes((0.2,0.85,0.1,0.05));
+    def butcancel(a=None,fig=fig):
+        """This is going to be called if the OK button is pressed"""
+        fig.smoothingdone=True
+        fig.cancel=True
+    but=matplotlib.widgets.Button(ax,'Cancel')
+    but.on_clicked(butcancel)
+
+    
+    axes=fig.add_axes((0.1,0.1,0.8,0.7))
+    def fun(a,fig=fig,axes=axes,xscale=xscaling,yscale=yscaling,x=x,y=y,sl=sl,smoothmode=smoothingmode,callback=callback,thisisthefirstplot=False):
+        ax=axes.axis()
+        axes.cla()
+        axes.plot(x,y,'.')
+        axes.set_xscale(xscale)
+        axes.set_yscale(yscale)
+        fig.smoothing=pow(10.0,sl.val);
+        fig.ysmoothed=fitting.smoothcurve(x,y,fig.smoothing,mode=smoothmode)
+        axes.plot(x,fig.ysmoothed,linewidth=2)
+        if not thisisthefirstplot:
+            axes.axis(ax)
+        if callback is not None:
+            callback(fig.smoothing,fig.ysmoothed,axes)
+        pylab.draw()
+        fig.show()
     sl.on_changed(fun)
-    fun(1e-5)
+    fun(1e-5,thisisthefirstplot=True)
     while not fig.smoothingdone:
         pylab.waitforbuttonpress()
-    pylab.clf()
-    pylab.plot(x,y,'.')
-    y1=fitting.smoothcurve(x,y,pow(10,sl.val),mode='spline')
-    pylab.plot(x,y1,linewidth=2)
-    pylab.draw()
-    return pow(10,sl.val)
+    if fig.cancel:
+        raise RuntimeError('Smoothing was cancelled')
+    if returnsmoothed:
+        return fig.smoothing,fig.ysmoothed
+    else:
+        return fig.smoothing
 
 def testorigin(data,orig,mask=None,dmin=None,dmax=None):
     """Shows several test plots by which the validity of the determined origin
@@ -954,13 +893,13 @@ def assesstransmission(fsns,titleofsample,mode='Gabriel',dirs=[]):
         pylab.grid('on')
         
     pylab.axes(axisTransm)
-    pylab.legend(loc='center left',bbox_to_anchor=(hspacing+hborder+axiswidth,1-vborder-0*vspacing-0.5*axisheight), bbox_transform=pylab.gcf().transFigure,prop={'size':'small'})
+    pylab.legend(loc='center left',bbox_to_anchor=(hspacing+hborder+axiswidth,1-vborder-0*vspacing-0.5*axisheight), bbox_transform=pylab.gcf().transFigure,prop={'size':'xx-small'})
     pylab.axes(axisBcX)
-    pylab.legend(loc='center left',bbox_to_anchor=(hspacing+hborder+axiswidth,1-vborder-1*vspacing-1.5*axisheight), bbox_transform=pylab.gcf().transFigure,prop={'size':'small'})
+    pylab.legend(loc='center left',bbox_to_anchor=(hspacing+hborder+axiswidth,1-vborder-1*vspacing-1.5*axisheight), bbox_transform=pylab.gcf().transFigure,prop={'size':'xx-small'})
     pylab.axes(axisBcY)
-    pylab.legend(loc='center left',bbox_to_anchor=(hspacing+hborder+axiswidth,1-vborder-2*vspacing-2.5*axisheight), bbox_transform=pylab.gcf().transFigure,prop={'size':'small'})
+    pylab.legend(loc='center left',bbox_to_anchor=(hspacing+hborder+axiswidth,1-vborder-2*vspacing-2.5*axisheight), bbox_transform=pylab.gcf().transFigure,prop={'size':'xx-small'})
     pylab.axes(axisDORIS)
-    pylab.legend(loc='center left',bbox_to_anchor=(hspacing+hborder+axiswidth,1-vborder-3*vspacing-3.5*axisheight), bbox_transform=pylab.gcf().transFigure,prop={'size':'small'})
+    pylab.legend(loc='center left',bbox_to_anchor=(hspacing+hborder+axiswidth,1-vborder-3*vspacing-3.5*axisheight), bbox_transform=pylab.gcf().transFigure,prop={'size':'xx-small'})
     
 def findpeak(xdata,ydata,prompt=None,mode='Lorentz',scaling='lin',blind=False,return_error=False):
     """GUI tool for locating peaks by zooming on them
@@ -1009,6 +948,7 @@ def findpeak(xdata,ydata,prompt=None,mode='Lorentz',scaling='lin',blind=False,re
     def lorentzcostfun(p,x,y):
         tmp=y-p[3]-p[0]*utils.lorentzian(p[2],p[1],x)
         return tmp
+    x2=np.linspace(x1.min(),x1.max(),len(x1)*10)
     if mode=='Gauss':
         sigma0=0.25*(x1[-1]-x1[0])
         p0=((y1.max()-y1.min())/(1/np.sqrt(2*np.pi*sigma0**2)),
@@ -1020,9 +960,9 @@ def findpeak(xdata,ydata,prompt=None,mode='Lorentz',scaling='lin',blind=False,re
         cov=res[1]
         if not blind:
             if scaling=='log':
-                pylab.semilogy(x1,p1[3]+p1[0]/(np.sqrt(2*np.pi)*p1[1])*np.exp(-(x1-p1[2])**2/(2*p1[1]**2)),'r-')
+                pylab.semilogy(x2,p1[3]+p1[0]/(np.sqrt(2*np.pi)*p1[1])*np.exp(-(x2-p1[2])**2/(2*p1[1]**2)),'r-')
             else:
-                pylab.plot(x1,p1[3]+p1[0]/(np.sqrt(2*np.pi)*p1[1])*np.exp(-(x1-p1[2])**2/(2*p1[1]**2)),'r-')
+                pylab.plot(x2,p1[3]+p1[0]/(np.sqrt(2*np.pi)*p1[1])*np.exp(-(x2-p1[2])**2/(2*p1[1]**2)),'r-')
     elif mode=='Lorentz':
         sigma0=0.25*(x1[-1]-x1[0])
         p0=((y1.max()-y1.min())/(1/sigma0),
@@ -1034,13 +974,14 @@ def findpeak(xdata,ydata,prompt=None,mode='Lorentz',scaling='lin',blind=False,re
         cov=res[1]
         if not blind:
             if scaling=='log':
-                pylab.semilogy(x1,p1[3]+p1[0]*utils.lorentzian(p1[2],p1[1],x1),'r-')
+                pylab.semilogy(x2,p1[3]+p1[0]*utils.lorentzian(p1[2],p1[1],x2),'r-')
             else:
-                pylab.plot(x1,p1[3]+p1[0]*utils.lorentzian(p1[2],p1[1],x1),'r-')
+                pylab.plot(x2,p1[3]+p1[0]*utils.lorentzian(p1[2],p1[1],x2),'r-')
     else:
         raise ValueError('Only Gauss and Lorentz modes are supported in findpeak()')
     if not blind:
         pylab.gcf().show()
+        pylab.draw()
     if return_error:
         return p1[2],np.sqrt(cov[2][2])
     else:
@@ -1339,3 +1280,57 @@ def azimsectorgui(data,dataerr,param,mask):
     doreset()
     while not fig.userdata['done']:
         fig.waitforbuttonpress()
+def assessmostab(fsns,dirs='.'):
+    """Draw graphs on the current figure to assess the status of the monochromator stabilisator.
+    
+    Inputs:
+        fsns: range of file sequence numbers
+        dirs: list of directories to search header files in.
+    """
+    header=B1io.readheader('org_',fsns,'.header',dirs)
+    fsn=[h['FSN'] for h in header]
+    monitor=[h['Monitor']/h['MeasTime'] for h in header]
+    doris=[h['MonitorDORIS']/h['MeasTime'] for h in header]
+    piezo=[h['MonitorPIEZO']/h['MeasTime'] for h in header]
+    energies=[h['Energy'] for h in header]
+    pylab.clf()
+    pylab.gcf().subplotpars.wspace=0.4
+    pylab.subplot(2,2,1);
+    pylab.plot(fsn,monitor,'b',label='Monitor')
+    pylab.ylabel('Monitor cps',color='blue')
+    pylab.xlabel('FSN')
+    pylab.twinx()
+    pylab.plot(fsn,doris,'r',label='DORIS counter')
+    pylab.ylabel('DORIS cps',color='red')
+    pylab.subplot(2,2,2);
+    pylab.plot(fsn,monitor,'b',label='Monitor')
+    pylab.ylabel('Monitor cps',color='blue')
+    pylab.xlabel('FSN')
+    pylab.twinx()
+    pylab.plot(fsn,piezo,'g',label='PIEZO counter')
+    pylab.ylabel('PIEZO cps',color='green')
+    pylab.subplot(2,2,3);
+    pylab.plot(fsn,doris,'r',label='DORIS counter')
+    pylab.ylabel('DORIS cps',color='red')
+    pylab.xlabel('FSN')
+    pylab.twinx()
+    pylab.plot(fsn,piezo,'g',label='PIEZO counter')
+    pylab.ylabel('PIEZO cps',color='green')
+    pylab.subplot(2,2,4);
+    pylab.plot(fsn,map(lambda a,b:a/b,piezo,doris),'k',label='ratio')
+    for e in utils.unique(energies):
+        pylab.plot([f for f,e1 in zip(fsn,energies) if e1==e],
+                   [p/d for p,d,e1 in zip(piezo,doris,energies) if e1==e],
+                   'o',label='%.2f'%e)
+    pylab.ylabel('Piezo/DORIS ratio',color='black')
+    pylab.xlabel('FSN')
+    pylab.twinx()
+    pylab.plot(fsn,map(lambda a,b:a/b,monitor,doris),'m')
+    for e in utils.unique(energies):
+        pylab.plot([f for f,e1 in zip(fsn,energies) if e1==e],
+                   [m/d for m,d,e1 in zip(monitor,doris,energies) if e1==e],
+                   'o',label='%.2f eV'%e)
+    pylab.ylabel('Monitor/DORIS ratio',color='magenta')
+    pylab.legend(loc='best',prop={'size':'x-small'})
+    pylab.gcf().show()
+    pylab.draw()
